@@ -49,7 +49,7 @@ static CorpusList *GetSystemCorpus(char *name, char *registry);
 /**
  * Global pointer to the head of CQP's linked list of the currently-loaded corpora (and NQRs).
  */
-static CorpusList *corpuslist = NULL;
+extern CorpusList *corpuslist;
 
 
 /**
@@ -1203,8 +1203,7 @@ GetSystemCorpus(char *name, char *registry)
 }
 
 
-
-static CorpusList *
+CorpusList *
 ensure_syscorpus(char *registry, char *name)
 {
   CorpusList *cl;
