@@ -1,5 +1,5 @@
 library(RcppCWB)
-
+use_tmp_registry()
 testthat::context("cl_id2str")
 
 test_that(
@@ -8,7 +8,7 @@ test_that(
     token <- cl_id2str(
       corpus = "REUTERS",
       p_attribute = "word",
-      registry = use_tmp_registry(),
+      registry = get_tmp_registry(),
       id = 0L:5L
     )
     expect_equal(token, c("Diamond", "Shamrock", "Corp", "said", "that", "effective"))
